#include-once
#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>
#include <GuiListView.au3>
#include <string.au3>
; #FUNCTION# ====================================================================================================================
; Name...........: _ArraySortMulti
; Description ...: Sort multiple or multi-dimensional arrays with multiple sort columns.
;                  Sort two arrays with the same dimensions at once or a multi-dimensional array by one or more dimensions.
;                  When multiple arrays are used only the first array has parameters, so that the second one will be
;                  sorted in the same way as the first one. Good when you need a user exit to sort an array in a special way.
; Syntax.........: _ArraySortMulti(ByRef $avArray .., COL0 ..[, mixed $arg], COL1 ..[, mixed $arg], [DEBUG])
; Parameters ....: $avArray	      - Arrays to sort separated by comma. Only one or two arrays allowed.
;                                   Multiple arrays must be of the same dimensions, but can contain any data.
;                                   Only the first array in the list will have parameters.
;                                   The second one will be sorted the same way as the first.
;                  $sCOL0,$sCOL1 	- Required. Must be COL followed by a number. First column start with zero - COL0.
;                                   A 1-dimensional array has one column named COL0. Multiple sort columns are allowed in any order.
;                                   This parameter cannot be a constant since you can have two array parameters with ByRef.
;                  $arguments    	- [optional] ASC, DESC, REGULAR, NUMERIC, STRING, INSEN_STRING, UserOrder, HexOrder.
;                                   Can be repeated for every column to sort. All arguments can be specified in any order
;                                   and in both lower and upper case. You can place all arguments in one string separated by comma
;                                   or mix it with multiple strings in separate variables. Do whatever is convenient to you.
;                  $sASC         	- Default=ASC. ASC gives Ascending sort order.
;                  $sDESC        	- Default=ASC. DESC gives Descending sort order.
;                  $sREGULAR     	- Default=REGULAR. REGULAR treat values as they are. Don't change types.
;                                   Lexicographically string compares. Case insensitive.
;                                   WARNING: Using REGULAR with strings might give you unpredictable results like 'AA' > 'AB'.
;                                   Use STRING, INSEN_STRING and NUMERIC whenever possible.
;                  $sNUMERIC     	- Default=REGULAR. NUMERIC treat values numerically.
;                  $sSTRING    	  - Default=REGULAR. STRING treat values as strings. Case sensitive.
;                  $sINSEN_STRING - Default=REGULAR. INSEN_STRING treat values as strings. Case insensitive.
;                  $sUserOrder    - Default=No order change. Change sort order without changing any data in the array.
;                                   Specify UserOrder=(DataFrom=DataTo in pairs (FromTo) separated by semicolon).
;                                   Example UserOrder=(January=1;February=2;March=3;April=4) Changes the orders of months lacking numbers.
;                                   You do not use single or double quotes inside parenthesis unless they are data themselves.
;                                   Parameters like NUMERIC, STRING applies like normal, see examples.
;                  $sHexOrder    	- Default=No order change. Change sort order without changing any data in the array.
;                                   Specify HexOrder=(HexFrom=HexTo in pairs (FromTo) separated by semicolon).
;                                   Example HexOrder=(C6=5B;D8=5C;C5=5D;E6=7B;F8=7C;E5=7D) Changes hexcode C6 to 5B and so on.
;                                   The main purpose of this is to simplify sorting for countries like the Scandinavians
;                                   who use additional letters to the english alphabet. A one time job for each country.
;                                   Example _StringToHex('') & _StringToHex(Chr(91)) vil give HexOrder=(C6=5B)
;                                   Replace hexcode C6 with 5B without changing data so as to change sort order of strings.
;                  $sDEBUG       	- Default=No debug. If DEBUG will list the arrays before and after sort. You only need to specify once.
; Return values .: Success = 1
;                  Failure = 0, sets @error:
;                  |1 - $avArray is not an array
;                  |2 - Multiple arrays is not of the same dimensions
;                  |3 - Invalid COL parameter
;                  |4 - COL parameter missing or in wrong position
;                  |5 - Unknown parameter found
;                  |6 - Max 9-dimensions exceeded. Can easily be extended on request.
;                  |7 - Cannot use NUMERIC and HexOrder on the same column
;                  |8 - HexOrder specifications invalid
;                  |9 - UserOrder specifications invalid
; Author ........: Thopaga
; Modified.......:
; Remarks .......: This routine focus on functionality and flexibility and not on performance.
;                  It is meant to be a supplement to the _ArraySort routine which focus is on performance.
; Related .......: _ArraySort
; Link ..........:
; Example .......: Yes
; ===============================================================================================================================
Func _ArraySortMulti(ByRef $avA1,ByRef $avA2, $sP1="",$sP2="",$sP3="",$sP4="",$sP5="",$sP6="",$sP7="",$sP8="",$sP9="",$sP10="", _
	$sP11="",$sP12="",$sP13="",$sP14="",$sP15="",$sP16="",$sP17="",$sP18="",$sP19="",$sP20="")
; ==========================================================
; Parameters.
; ==========================================================
	If Not IsArray($avA1) Then Return SetError(1, 0, 0) ; $avArray is not an array

; Define variables and arrays.
	Local $iDim=0, $iDimCol=0, $fDebug=False, $iSortCol=-1
	Local $iCount=0, $iStart=0, $iEnd=0, $fMulti=False, $sP0="", $sParam="", $asVar="", $sVar="", $iCol=0, $iAdd=0, $iTotal=0
	Local $sIE="", $sJE="", $sIEValue="", $sJEValue="",$k=0, $sHexFrom="", $sHexTo="", $iVar1, $iVar2, $vVar1, $vVar2
	Local $iUserElement=0, $iPos=0, $iPos1=0, $iPos2=0, $sTemp=""
	$iDim 	 = UBound($avA1,0)
	$iDimCol = UBound($avA1,$iDim)
	Local $aiSumElement[$iDim]
	Local $aiSortCol[$iDimCol]
	Local $afASC[$iDimCol]
	Local $asNumString[$iDimCol]
	Local $asHexOrder[$iDimCol][2]
	Local $asParams[1]
	Local $asUserOrder[$iDimCol][3]
	Local $asUserWork[1]
	Local $asUserData[1][2]

	; Calculate total elements for each subscript.
	;Example Array[5][4][3] will give  $aiSumElement = [5*4*3 ,4*3 ,3]
	$iCount = 0
	For $i = $iDim To 1 Step -1
		If $iCount = 0 Then
			$iCount = UBound($avA1,$i)
		Else
			$iCount = $iCount * UBound($avA1,$i)
		EndIf
		$aiSumElement[$I-1] = $iCount
	Next

; Read parameters and put them in an array.
	$iStart = 1
	$iEnd 	= @NumParams-2
	$fMulti = True
	If Not IsArray($avA2) Then
		$sP0=$avA2
		$iStart = 0
		$avA2 = ""
		$fMulti = False
	EndIf
	If $fMulti And UBound($avA1,0) <> UBound($avA2,0) Then Return SetError(2, 0, 0) ; Multiple arrays not same dimensions
	If $fMulti Then
		For $i = 1 To UBound($avA1,0)
			If UBound($avA1,$i) <> UBound($avA2,$i) Then Return SetError(2, 0, 0) ; Multiple arrays not same dimensions
		Next
	EndIf
	$iDim = UBound($avA1,0)

	$k=0
	For $i = $iStart To $iEnd
		$sParam = Eval("sP" & $i)
		If StringInStr($sParam, "UserOrder=(") > 0 Then
			$ipos1 = StringInStr($sParam, "UserOrder=(")
			If $ipos1 > 0 Then $ipos1 = 9
			$ipos2 = StringInStr($sParam,")",0,1,$ipos1)
			If $ipos2 > $ipos1 And $ipos1 > 0 Then
				$sTemp = StringMid($sParam, $ipos1, $ipos2-$ipos1)
				If StringInStr($sTemp, ",") > 0 Then Return SetError(9, 0, 0) ; UserOrder specifications invalid
			EndIf
		EndIf
		If StringInStr($sParam, "HexOrder=(") > 0 Then
			$ipos1 = StringInStr($sParam, "HexOrder=(")
			If $ipos1 > 0 Then $ipos1 = 8
			$ipos2 = StringInStr($sParam,")",0,1,$ipos1)
			If $ipos2 > $ipos1 And $ipos1 > 0 Then
				$sTemp = StringMid($sParam, $ipos1, $ipos2-$ipos1)
				If StringInStr($sTemp, ",") > 0 Then Return SetError(8, 0, 0) ; HexOrder specifications invalid
			EndIf
		EndIf
		If StringInStr($sParam, ",") > 0 Then
			$asVar = StringSplit($sParam,",")
			For $j = 1 To $asVar[0]
				$k=$k+1
				ReDim $asParams[$k]
				$asParams[$k-1] =  $asVar[$j]
			Next
		Else
			$k=$k+1
			ReDim $asParams[$k]
			$asParams[$k-1] =  $sParam
		EndIf
	Next
	; All parameters collected.
	; Read them, check them and put them in their respective arrays.
	For $i = 0 To $k-1
		$sParam = $asParams[$i]
		If StringInStr($sParam, "COL") > 0 Then
			$iCol = Number(StringMid($sParam, 4))
			If $iDim = 1 Then
				If Not IsInt($iCol) Or $iCol > 0 Then Return SetError(3, 0, 0) ; Invalid COL parameter
			Else
				If Not IsInt($iCol) Or $iCol > UBound($avA1,$iDim)-1 Then Return SetError(3, 0, 0) ; Invalid COL parameter
			EndIf
			$iSortCol=$iSortCol+1
			$aiSortCol[$iSortCol] = $iCol
			$afASC[$iSortCol] = True
			$asNumString[$iSortCol] = "REGULAR"
			$asHexOrder[$iSortCol][0] = False
			$asUserOrder[$iSortCol][0] = False
			ContinueLoop
		EndIf
		If $iSortCol < 0 Then Return SetError(4, 0, 0) ; COL parameter missing or in wrong position
		If StringInStr($sParam, "HexOrder=(") > 0 Then
			$sVar = StringReplace($sParam, "HexOrder=(", "")
			$sVar = StringReplace($sVar, ")", "")
			$sVar = StringReplace($sVar, ";", "")
			$sVar = StringReplace($sVar, "=", "")
			$sVar = StringReplace($sVar, " ", "")
			$asHexOrder[$iSortCol][0] = True
			$asHexOrder[$iSortCol][1] = $sVar
			If $asNumString[$iSortCol] = "NUMERIC" Then Return SetError(7, 0, 0) ; Cannot use NUMERIC and HexOrder on the same column
			If Not StringIsXDigit($sVar) Then Return SetError(8, 0, 0) ; HexOrder specifications invalid
			If StringLen($sVar)/4 - Floor(StringLen($sVar)/4) > 0  Then Return SetError(8, 0, 0) ; HexOrder specifications invalid
			ContinueLoop
		EndIf
		If StringInStr($sParam, "UserOrder=(") > 0 Then
			$sVar = StringReplace($sParam, "UserOrder=(", "")
			$sVar = StringReplace($sVar, ")", "")
			$asUserOrder[$iSortCol][0] = True
			$asUserOrder[$iSortCol][2] = $sVar
			If StringInStr($sVar,"=") = 0 Then Return SetError(9, 0, 0) ; UserOrder specifications invalid
			$asUserWork = StringSplit($sVar, "=")
			$iUserElement = $asUserWork[0]-1
			$iCount=1
			If StringInStr($sVar,";") > 0 Then
				$asUserWork = StringSplit($sVar, ";")
				$iCount = $asUserWork[0]
			EndIf
			If $iUserElement <> $iCount Then Return SetError(9, 0, 0) ; UserOrder specifications invalid
			$asUserOrder[$iSortCol][1] = $iUserElement
			ContinueLoop
		EndIf
		Select
			Case $sParam = "ASC"
				$afASC[$iSortCol] = True
			Case $sParam = "DESC"
				$afASC[$iSortCol] = False
			Case $sParam = "REGULAR"
				$asNumString[$iSortCol] = "REGULAR"
			Case $sParam = "NUMERIC"
				$asNumString[$iSortCol] = "NUMERIC"
				If $asHexOrder[$iSortCol][0] Then Return SetError(7, 0, 0) ; Cannot use NUMERIC and HexOrder on the same column
			Case $sParam = "STRING"
				$asNumString[$iSortCol] = "STRING"
			Case $sParam = "INSEN_STRING"
				$asNumString[$iSortCol] = "INSEN_STRING"
			Case $sParam = "DEBUG"
				$fDebug = True
			Case Else
				;MsgBox(0,"","$sParam=" & $sParam)
				Return SetError(5, 0, 0) ; Unknown parameter found
		EndSelect
	Next
	If $iDim > 9 Then Return SetError(6, 0, 0) ; Max 9-dimensions exceeded

; ============================================================================================================
; Main loop and processing.
; Read each sort column parameter and loop through the whole array.
; Swap elements according to sort specifications.
; If this is the second or more column parameter then exit inner loop if not equal elements on previous column.
; ============================================================================================================
If $fDebug Then _MultiDisplay("BEFORE - List of primary array before sort","$avA1",$iDim,$avA1,$aiSumElement)
If $fDebug And $fMulti Then _MultiDisplay("BEFORE - List of second array before sort","$avA2",$iDim,$avA2,$aiSumElement)

If $iDim = 1 Then
	$iAdd = 1
Else
	$iAdd = $aiSumElement[$iDim-1]
EndIf
$iTotal = $aiSumElement[0]-1

For $iCol = 0 To $iSortCol
	$iSortCol = $aiSortCol[$iCol]
	If $asUserOrder[$iCol][0] Then
		If $asUserOrder[$iCol][1] > 1 Then
			$asUserWork = StringSplit($asUserOrder[$iCol][2], ";",2)
		Else
			ReDim $asUserWork[1]
			$asUserWork[0] = $asUserOrder[$iCol][2]
		EndIf
		Redim $asUserData[$asUserOrder[$iCol][1]][2]
		For $k = 0 To $asUserOrder[$iCol][1]-1
			$iPos = StringInStr($asUserWork[$k],"=")
			$asUserData[$k][0] = StringMid($asUserWork[$k],1,$iPos-1)
			$asUserData[$k][1] = StringMid($asUserWork[$k],$iPos+1)
		Next
		If $asNumString[$iCol] = "NUMERIC" Then
			For $k = 0 To $asUserOrder[$iCol][1]-1
				$asUserData[$k][1] = Number($asUserData[$k][1])
			Next
		EndIf
	EndIf
	For $i = $iSortCol To $iTotal-1 Step $iAdd
		$sIE = _MultiElement($iSortCol, $i, $iDim, $aiSumElement)
		For $j = $i+$iAdd To $iTotal Step $iAdd
			$sIEValue = Execute($sIE)
			$sJE = _MultiElement($iSortCol, $j, $iDim, $aiSumElement)
			$sJEValue = Execute($sJE)
			If $iCol > 0 Then
				$iVar1 = $j - $iSortCol + $aiSortCol[$iCol-1]
				$vVar1 = _MultiElement($aiSortCol[$iCol-1], $iVar1, $iDim, $aiSumElement)
				$iVar2 = $j - $iSortCol + $aiSortCol[$iCol-1] - $iAdd
				$vVar2 = _MultiElement($aiSortCol[$iCol-1], $iVar2, $iDim, $aiSumElement)
				If $asNumString[$iCol-1] = "STRING" Then
					If StringCompare(Execute($vVar1),Execute($vVar2),1) <> 0 Then ExitLoop
				Else
					If Execute($vVar1) <> Execute($vVar2) Then ExitLoop
				EndIf
			EndIf
			If $asUserOrder[$iCol][0] Then
				For $k = 0 To $asUserOrder[$iCol][1]-1
					Select
						Case $asNumString[$iCol] = "STRING"
							If StringCompare($sIEValue,$asUserData[$k][0],1) = 0 Then $sIEValue = $asUserData[$k][1]
							If StringCompare($sJEValue,$asUserData[$k][0],1) = 0 Then $sJEValue = $asUserData[$k][1]
						Case $asNumString[$iCol] = "INSEN_STRING"
							If StringCompare($sIEValue,$asUserData[$k][0],2) = 0 Then $sIEValue = $asUserData[$k][1]
							If StringCompare($sJEValue,$asUserData[$k][0],2) = 0 Then $sJEValue = $asUserData[$k][1]
						Case Else
							If $sIEValue = $asUserData[$k][0] Then $sIEValue = $asUserData[$k][1]
							If $sJEValue = $asUserData[$k][0] Then $sJEValue = $asUserData[$k][1]
					EndSelect
				Next
			EndIf
			If $asHexOrder[$iCol][0] Then
				For $k = 1 To StringLen($asHexOrder[$iCol][1]) Step 4
					$sHexFrom = _HexToString(StringMid($asHexOrder[$iCol][1],$k,2))
					$sHexTo   = _HexToString(StringMid($asHexOrder[$iCol][1],$k+2,2))
					$sIEValue = StringReplace($sIEValue, $sHexFrom, $sHexTo, 0, 1)
					$sJEValue = StringReplace($sJEValue, $sHexFrom, $sHexTo, 0, 1)
				Next
			EndIf
			Select
				Case $asNumString[$iCol] = "STRING"
					If   (StringCompare($sIEValue,$sJEValue,1) > 0 And $afASC[$iCol]) _
						Or (StringCompare($sIEValue,$sJEValue,1) < 0 And Not $afASC[$iCol]) _
						Then _MultiSwap($avA1,$avA2,$fMulti,$i,$j,$iSortCol,$iDim,$aiSumElement)
				Case $asNumString[$iCol] = "INSEN_STRING"
					If   (StringCompare($sIEValue,$sJEValue,2) > 0 And $afASC[$iCol]) _
						Or (StringCompare($sIEValue,$sJEValue,2) < 0 And Not $afASC[$iCol]) _
						Then _MultiSwap($avA1,$avA2,$fMulti,$i,$j,$iSortCol,$iDim,$aiSumElement)
				Case $asNumString[$iCol] = "NUMERIC"
					If 	 (Number($sIEValue) > Number($sJEValue) And $afASC[$iCol]) _
						Or (Number($sIEValue) < Number($sJEValue) And Not $afASC[$iCol]) _
						Then _MultiSwap($avA1,$avA2,$fMulti,$i,$j,$iSortCol,$iDim,$aiSumElement)
				Case Else
					If 	 ($sIEValue > $sJEValue And $afASC[$iCol]) _
						Or ($sIEValue < $sJEValue And Not $afASC[$iCol]) _
						Then _MultiSwap($avA1,$avA2,$fMulti,$i,$j,$iSortCol,$iDim,$aiSumElement)
			EndSelect
		Next
	Next
Next

If $fDebug Then _MultiDisplay("AFTER - List of primary array after sort","$avA1",$iDim,$avA1,$aiSumElement)
If $fDebug And $fMulti Then _MultiDisplay("AFTER - List of second array after sort","$avA2",$iDim,$avA2,$aiSumElement)
Return 1
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __MultiElement
; Description ...: Helper function for _ArraySortMulti.
;                  Calculate element in the array based on relative address.
; Syntax.........: __MultiElement($iSortCol, $iIndex, $iDim, ByRef $aiSumElement)
; Parameters ....: $iSortCol 			- Column to sort.
;                  $iIndex   			- Relative address of element.
;                  $iDim     			- Dimension of the array.
;                  $aiSumElement 	- Array that contains total number of elements for each subscripts.
;                  									Example Array[5][4][3] will give  $aiSumElement = [5*4*3 ,4*3 ,3]
; Return values .: Calculated element with subscripts.
; Author ........: Thopaga
; Modified.......:
; Remarks .......: For Internal Use Only
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _MultiElement($iSortCol, $iIndex, $iDim, ByRef $aiSumElement)
	Local $sElement, $sVar, $i
	$sElement = "$avA1"
	For $i = 0 To $iDim-1
		If $i < $iDim-1 Then
				If $i > 0 Then
					$sVar = Floor(($iIndex-Floor($iIndex/$aiSumElement[$i]) * $aiSumElement[$i])/$aiSumElement[$i+1])
				Else
					$sVar = Floor($iIndex/$aiSumElement[$i+1])
				EndIf
		Else
			$sVar = $iSortCol
			If $iDim = 1 Then $sVar = $iIndex
		EndIf
		$sElement = $sElement & "[" & $sVar & "]"
	Next
	Return($sElement)
EndFunc

; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __MultiSwap
; Description ...: Helper function for _ArraySortMulti.
;									 Swap all elements in the last subscript.
; Syntax.........: _MultiSwap(Byref $avA1, Byref $avA2, $fMulti, $i, $j, $iSortCol, $iDim, ByRef $aiSumElement)
; Parameters ....: $avA1					- Primary array.
;                  $avA2  				- Second array when multiple arrays used.
;                  $fMulti     		- Multiple array used - True or False.
;                  $i     				- Relative address of element to swap.
;                  $j     				- Relative address of the other element to swap.
;                  $iSortCol    	- Column to sort.
;                  $iDim    			- Dimension of the array.
;                  $aiSumElement 	- Array that contains total number of elements for each subscripts.
;                  									Example Array[5][4][3] will give  $aiSumElement = [5*4*3 ,4*3 ,3]
; Return values .: None
; Author ........: Thopaga
; Modified.......:
; Remarks .......: For Internal Use Only
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _MultiSwap(Byref $avA1, Byref $avA2, $fMulti, $i, $j, $iSortCol, $iDim, ByRef $aiSumElement)
	Local $aI[$iDim]
  Local $aJ[$iDim]
	Local  $sIE, $sJE, $iE, $TMP

	; Loop through all elements of the last subscript and calculate the element based on relative address.
	$i = $i - $iSortCol
	$j = $j - $iSortCol
	For $iE = 0 to $aiSumElement[$iDim-1]-1
		$sIE = "$avA1"
		$sJE = "$avA1"
		For $k = 0 To $iDim-1
			If $k < $iDim-1 Then
				If $k > 0 Then
					$aI[$k] = Floor(($I-Floor($I/$aiSumElement[$k]) * $aiSumElement[$k])/$aiSumElement[$k+1])
					$aJ[$k] = Floor(($J-Floor($J/$aiSumElement[$k]) * $aiSumElement[$k])/$aiSumElement[$k+1])
				Else
					$aI[$k] = Floor($I/$aiSumElement[$k+1])
				  $aJ[$k] = Floor($J/$aiSumElement[$k+1])
				EndIf
			Else
				$aI[$k] = $iE
				$aJ[$k] = $iE
				If $iDim = 1 Then $aI[$k] = $i
				If $iDim = 1 Then $aJ[$k] = $j
			EndIf
			$sIE = $sIE & "[" & $aI[$k] & "]"
			$sJE = $sJE & "[" & $aJ[$k] & "]"
		Next
		; Swap according to dimension.
		Select
			Case $iDim = 1
				$TMP  			  = Execute($sIE)
				$avA1[$aI[0]] = Execute($sJE)
				$avA1[$aJ[0]] = $TMP
			Case $iDim = 2
				$TMP  					   = Execute($sIE)
				$avA1[$aI[0]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$iE] = $TMP
			Case $iDim = 3
				$TMP  									 	 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$iE] = $TMP
			Case $iDim = 4
				$TMP  							 							 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$iE] = $TMP
			Case $iDim = 5
				$TMP  									 		 							 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$iE] = $TMP
			Case $iDim = 6
				$TMP  									 		 											 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$iE] = $TMP
			Case $iDim = 7
				$TMP  									 		 											 				 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$iE] = $TMP
			Case $iDim = 8
				$TMP  									 		 																 			 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$aI[6]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$aJ[6]][$iE] = $TMP
			Case $iDim = 9
				$TMP  									 		 																 							 = Execute($sIE)
				$avA1[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$aI[6]][$aI[7]][$iE] = Execute($sJE)
				$avA1[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$aJ[6]][$aJ[7]][$iE] = $TMP
		EndSelect

		; Swap on the second array.
		If $fMulti Then
			$sIE = StringReplace($sIE, "$avA1", "$avA2")
			$sJE = StringReplace($sJE, "$avA1", "$avA2")
			Select
				Case $iDim = 1
					$TMP  			  = Execute($sIE)
					$avA2[$aI[0]] = Execute($sJE)
					$avA2[$aJ[0]] = $TMP
				Case $iDim = 2
					$TMP  					   = Execute($sIE)
					$avA2[$aI[0]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$iE] = $TMP
				Case $iDim = 3
					$TMP  									 	 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$iE] = $TMP
				Case $iDim = 4
					$TMP  														 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$iE] = $TMP
				Case $iDim = 5
					$TMP  									 		 							 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$iE] = $TMP
				Case $iDim = 6
					$TMP  									 		 											 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$iE] = $TMP
				Case $iDim = 7
					$TMP  									 		 															 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$iE] = $TMP
				Case $iDim = 8
					$TMP  									 		 												 							 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$aI[6]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$aJ[6]][$iE] = $TMP
				Case $iDim = 9
					$TMP  									 		 											 					 							 = Execute($sIE)
					$avA2[$aI[0]][$aI[1]][$aI[2]][$aI[3]][$aI[4]][$aI[5]][$aI[6]][$aI[7]][$iE] = Execute($sJE)
					$avA2[$aJ[0]][$aJ[1]][$aJ[2]][$aJ[3]][$aJ[4]][$aJ[5]][$aJ[6]][$aJ[7]][$iE] = $TMP
			EndSelect
		EndIf
		If $iDim = 1 Then Return
	Next
	Return
EndFunc


; #INTERNAL_USE_ONLY# ===========================================================================================================
; Name...........: __MultiDisplay
; Description ...: Helper function for _ArraySortMulti.
;									 ListView the whole array when DEBUG parameter used. Button for copy data to Clipboard.
; Syntax.........: _MultiDisplay($sHeading,$sArrayName, $iDim, ByRef $avA1, ByRef $aiSumElement)
; Parameters ....: $sHeading			- Heading text used in ListView.
;									 $sArrayName		- Name of array to list.
;                  $iDim    			- Dimension of the array.
;                  $avA1					- Array to list.
;                  $aiSumElement 	- Array that contains total number of elements for each subscripts.
;                  									Example Array[5][4][3] will give  $aiSumElement = [5*4*3 ,4*3 ,3]
; Return values .: None
; Author ........: Thopaga
; Modified.......:
; Remarks .......: For Internal Use Only
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _MultiDisplay($sHeading,$sArrayName, $iDim, ByRef $avA1, ByRef $aiSumElement)
	Local $sElement, $sVar, $i, $j, $sLine, $iAdd, $iCount, $Button1, $nMsg, $sLvw, $vElementValue, $iWidth, $sRow, $iRow=0

	; Make a ListView heading based on number of columns.
	; Do the same for ClipBoard.
	If $iDim = 1 Then
		$iAdd = 1
		$j = ($aiSumElement[0]/$aiSumElement[$iDim-1]) + 1
		$i = $aiSumElement[$iDim-1] + 2
	Else
		$iAdd =  $aiSumElement[$iDim-1]
		$i = ($aiSumElement[0]/$aiSumElement[$iDim-1]) + 2
		$j = $aiSumElement[$iDim-1] + 1
	EndIf
	Local $avClip[$i][$j]
	For $i = 0 To $j-1
		$avClip[0][$i] = 0
	Next
	$sLine = ""
	$iCount = 0
	For  $i = 0 To $aiSumElement[$iDim-1]-1
		If $i = 0 Then
			$sLine = "Row|"
			$iRow = $iRow + 1
			$avClip[$iRow][$i] = "ROW"
			$avClip[0][$i] = 6
		EndIf
		$sLine = $sLine & "Col " & $i & "|"
		$avClip[$iRow][$i+1] = "COL" & $i
		$avClip[0][$i+1] = 6
		If $iDim = 1 Then ExitLoop
	Next
	GUICreate($sHeading, 810, 650)
	$sLvw = GUICtrlCreateListView($sLine, 5, 5, 800, 600,$LVS_REPORT,$LVS_EX_GRIDLINES)

	; Loop through the whole array and calculate both Element and Elementvalue.
	; Keep track of string lengths for Clipboard.
	$sLine = ""
	For $i = 0 To $aiSumElement[0]-1
		$iCount = $iCount+1
		$sElement = "$avA1"
		For $j = 0 To $iDim-1
			If $j < $iDim-1 Then
				If $j > 0 Then
					$sVar = Floor(($i-Floor($I/$aiSumElement[$j]) * $aiSumElement[$j])/$aiSumElement[$j+1])
				Else
					$sVar = Floor($i/$aiSumElement[$j+1])
				EndIf
			Else
				$sVar = $i - Floor($i/$aiSumElement[$j]) * $aiSumElement[$j]
			EndIf
			If $j = $iDim-1  And $iDim > 1 Then
				$sRow = $sElement & "[" & "Col" & "]"
			Else
				$sRow = $sElement & "[" & $sVar & "]"
			EndIf
			$sElement = $sElement & "[" & $sVar & "]"
		Next
		$vElementValue = Execute($sElement)
		$iWidth = StringLen($vElementValue) + 2
		If $iWidth < 9 Then $iWidth = 9
		If IsNumber($vElementValue) Then
			$vElementValue = Stringformat("%" & $iWidth & "s", $vElementValue);
		Else
			$vElementValue = Stringformat("%-" & $iWidth & "s", $vElementValue);
		EndIf
		$sElement = $sRow
		If $sArrayName = "$avA2" Then $sElement = StringReplace($sElement, "$avA1", "$avA2")
		$iWidth = StringLen($sElement) + 2
		$sElement = Stringformat("%-" & $iWidth & "s", $sElement);
		; Put the item into ListView and Clipboard array.
		If $iDim=1 Or $iCount=1 Then $iRow = $iRow + 1
		If $iDim=1 Then
			$sLine = $sLine & $sElement & "|" & $vElementValue & "|"
			GUICtrlCreateListViewItem($sLine, $sLvw)
			$sLine = ""
			$avClip[$iRow][0] = $sElement
			$avClip[$iRow][1] = $vElementValue
			If StringLen($avClip[$iRow][0]) > $avClip[0][0] Then $avClip[0][0] = StringLen($avClip[$iRow][0])
			If StringLen($avClip[$iRow][1]) > $avClip[0][1] Then $avClip[0][1] = StringLen($avClip[$iRow][1])
		Else
			$avClip[$iRow][$iCount] = $vElementValue
			If StringLen($avClip[$iRow][$iCount]) > $avClip[0][$iCount] Then $avClip[0][$iCount] = StringLen($avClip[$iRow][$iCount])
			If $iCount = $iAdd Then
				$sLine = $sElement & "|" & $sLine & $vElementValue
				GUICtrlCreateListViewItem($sLine, $sLvw)
				$sLine = ""
				$iCount=0
				$avClip[$iRow][0] = $sElement
				If StringLen($avClip[$iRow][0]) > $avClip[0][0] Then $avClip[0][0] = StringLen($avClip[$iRow][0])
			Else
				$sLine = $sLine & $vElementValue & "|"
			EndIf
		EndIf
	Next

	; Autosize all columns in ListView
	For  $i = 0 To $aiSumElement[$iDim-1]
		_GUICtrlListView_SetColumnWidth($sLvw, $i, $LVSCW_AUTOSIZE)
		If $iDim = 1 And $i = 1 Then ExitLoop
	Next
	$Button1 = GUICtrlCreateButton("Copy to Clipboard", 350, 616, 163, 25, $WS_GROUP)
	GUISetState(@SW_SHOW)

	; Test for ListView action.
	While 1
		$nMsg = GUIGetMsg()
		Switch $nMsg
			Case $GUI_EVENT_CLOSE
				ExitLoop
			Case $Button1
				For $j = 0 To UBound($avClip,2)-1
					If Number($avClip[2][$j]) > 0 Then
						$avClip[1][$j] = StringFormat("%" & $avClip[0][$j] & "s", $avClip[1][$j])
					EndIf
				Next
				$sLine = ""
				For $i = 1 To UBound($avClip,1)-1
					For $j = 0 To UBound($avClip,2)-1
						$sLine = $sLine & StringFormat("%-" & $avClip[0][$j]+1 & "s", $avClip[$i][$j])
					Next
					$sLine = $sLine & @CRLF
				Next
				ClipPut($sLine)
		EndSwitch
	WEnd
	Return
EndFunc